# MIT LICENSE
#
# Copyright 1997 - 2019 by IXIA Keysight
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
from ixnetwork_restpy.base import Base
from ixnetwork_restpy.files import Files


class Capture(Base):
	"""The Capture class encapsulates a required capture node in the ixnetwork hierarchy.

	An instance of the class can be obtained by accessing the Capture property from a parent instance.
	The internal properties list will contain one and only one set of properties which is populated when the property is accessed.
	"""

	_SDM_NAME = 'capture'

	def __init__(self, parent):
		super(Capture, self).__init__(parent)

	@property
	def CurrentPacket(self):
		"""An instance of the CurrentPacket class.

		Returns:
			obj(ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.capture.currentpacket.currentpacket.CurrentPacket)

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		from ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.capture.currentpacket.currentpacket import CurrentPacket
		return CurrentPacket(self)._select()

	@property
	def Filter(self):
		"""An instance of the Filter class.

		Returns:
			obj(ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.capture.filter.filter.Filter)

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		from ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.capture.filter.filter import Filter
		return Filter(self)._select()

	@property
	def FilterPallette(self):
		"""An instance of the FilterPallette class.

		Returns:
			obj(ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.capture.filterpallette.filterpallette.FilterPallette)

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		from ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.capture.filterpallette.filterpallette import FilterPallette
		return FilterPallette(self)._select()

	@property
	def Trigger(self):
		"""An instance of the Trigger class.

		Returns:
			obj(ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.capture.trigger.trigger.Trigger)

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		from ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.capture.trigger.trigger import Trigger
		return Trigger(self)._select()

	@property
	def AfterTriggerFilter(self):
		"""Controls the capture of data after triggering when operating in triggered mode.

		Returns:
			str(captureAfterTriggerAll|captureAfterTriggerConditionFilter|captureAfterTriggerFilter)
		"""
		return self._get_attribute('afterTriggerFilter')
	@AfterTriggerFilter.setter
	def AfterTriggerFilter(self, value):
		self._set_attribute('afterTriggerFilter', value)

	@property
	def BeforeTriggerFilter(self):
		"""Controls the capture of data prior to triggering when operating in triggered mode

		Returns:
			str(captureBeforeTriggerAll|captureBeforeTriggerFilter|captureBeforeTriggerNone)
		"""
		return self._get_attribute('beforeTriggerFilter')
	@BeforeTriggerFilter.setter
	def BeforeTriggerFilter(self, value):
		self._set_attribute('beforeTriggerFilter', value)

	@property
	def CaptureMode(self):
		"""Controls whether data capture is performed in a continuous or triggered mode.

		Returns:
			str(captureContinuousMode|captureTriggerMode)
		"""
		return self._get_attribute('captureMode')
	@CaptureMode.setter
	def CaptureMode(self, value):
		self._set_attribute('captureMode', value)

	@property
	def ContinuousFilters(self):
		"""Controls the circular buffer behaviour: continuous capture of all received packets or continuous capture of received packets which match the filter conditions applied.

		Returns:
			str(captureContinuousAll|captureContinuousFilter)
		"""
		return self._get_attribute('continuousFilters')
	@ContinuousFilters.setter
	def ContinuousFilters(self, value):
		self._set_attribute('continuousFilters', value)

	@property
	def ControlActiveCapture(self):
		"""The name of the active control capture (if any).The active control capture is the last one made on the port by default; but the user can change it using this attribute.

		Returns:
			str
		"""
		return self._get_attribute('controlActiveCapture')
	@ControlActiveCapture.setter
	def ControlActiveCapture(self, value):
		self._set_attribute('controlActiveCapture', value)

	@property
	def ControlBufferBehaviour(self):
		"""Sets the control capture buffer behavior.

		Returns:
			str(bufferAfterStopCircular|bufferAfterStopNonCircular|bufferLiveCircular|bufferLiveNonCircular)
		"""
		return self._get_attribute('controlBufferBehaviour')
	@ControlBufferBehaviour.setter
	def ControlBufferBehaviour(self, value):
		self._set_attribute('controlBufferBehaviour', value)

	@property
	def ControlBufferSize(self):
		"""Sets the size(%) of the ports memory used by the control capture.

		Returns:
			number
		"""
		return self._get_attribute('controlBufferSize')
	@ControlBufferSize.setter
	def ControlBufferSize(self, value):
		self._set_attribute('controlBufferSize', value)

	@property
	def ControlCaptureFilter(self):
		"""Controls the dividing line within the capture buffer between before trigger dataand post trigger data. This control is only useful in triggered mode.

		Returns:
			str
		"""
		return self._get_attribute('controlCaptureFilter')
	@ControlCaptureFilter.setter
	def ControlCaptureFilter(self, value):
		self._set_attribute('controlCaptureFilter', value)

	@property
	def ControlCaptureState(self):
		"""Current state of the control capture (if there are packets uploading in GUI or not).

		Returns:
			str(notReady|ready)
		"""
		return self._get_attribute('controlCaptureState')

	@property
	def ControlCaptureTrigger(self):
		"""This is the control Trigger string.

		Returns:
			str
		"""
		return self._get_attribute('controlCaptureTrigger')
	@ControlCaptureTrigger.setter
	def ControlCaptureTrigger(self, value):
		self._set_attribute('controlCaptureTrigger', value)

	@property
	def ControlCapturedPacketCounter(self):
		"""

		Returns:
			number
		"""
		return self._get_attribute('controlCapturedPacketCounter')

	@property
	def ControlCaptures(self):
		"""The list of control captures which are available for the port.

		Returns:
			str
		"""
		return self._get_attribute('controlCaptures')

	@property
	def ControlDecodeAsCurrentFilter(self):
		"""The control capture decode as filter used by last decode as operation (if any).

		Returns:
			str
		"""
		return self._get_attribute('controlDecodeAsCurrentFilter')

	@property
	def ControlInterfaceType(self):
		"""Enables control capture on the desired interfaces.

		Returns:
			str(anyInterface|specificInterface)
		"""
		return self._get_attribute('controlInterfaceType')
	@ControlInterfaceType.setter
	def ControlInterfaceType(self, value):
		self._set_attribute('controlInterfaceType', value)

	@property
	def ControlPacketCounter(self):
		"""Shows the number of control capture packets.

		Returns:
			number
		"""
		return self._get_attribute('controlPacketCounter')

	@property
	def ControlSliceSize(self):
		"""Sets the size of the control capture slices.

		Returns:
			number
		"""
		return self._get_attribute('controlSliceSize')
	@ControlSliceSize.setter
	def ControlSliceSize(self, value):
		self._set_attribute('controlSliceSize', value)

	@property
	def DataActiveCapture(self):
		"""The name of the active data capture (if any). The active data capture is the last one made on the port by default; but the user can change it using this attribute.

		Returns:
			str
		"""
		return self._get_attribute('dataActiveCapture')
	@DataActiveCapture.setter
	def DataActiveCapture(self, value):
		self._set_attribute('dataActiveCapture', value)

	@property
	def DataCaptureState(self):
		"""Current state of the data capture; ready if all packets have been uploaded on client or notReady if packet uploading is in progress.

		Returns:
			str(notReady|ready)
		"""
		return self._get_attribute('dataCaptureState')

	@property
	def DataCapturedPacketCounter(self):
		"""

		Returns:
			number
		"""
		return self._get_attribute('dataCapturedPacketCounter')

	@property
	def DataCaptures(self):
		"""The list of data captures which are available for the port.

		Returns:
			str
		"""
		return self._get_attribute('dataCaptures')

	@property
	def DataDecodeAsCurrentFilter(self):
		"""The data capture decode as filter used by last decode as operation (if any).

		Returns:
			str
		"""
		return self._get_attribute('dataDecodeAsCurrentFilter')

	@property
	def DataPacketCounter(self):
		"""Shows the number of data capture packets.

		Returns:
			number
		"""
		return self._get_attribute('dataPacketCounter')

	@property
	def DataReceiveTimestamp(self):
		"""Controls whether the data capture packets timestamp are using the chassis UTC time or the HW timestamp.

		Returns:
			str(chassisUtcTime|hwTimestamp)
		"""
		return self._get_attribute('dataReceiveTimestamp')
	@DataReceiveTimestamp.setter
	def DataReceiveTimestamp(self, value):
		self._set_attribute('dataReceiveTimestamp', value)

	@property
	def DecodeAsLinkProtocols(self):
		"""List with link protocols available for capture decode as operation. Need to have an active capture to retrieve the property.

		Returns:
			list(str)
		"""
		return self._get_attribute('decodeAsLinkProtocols')

	@property
	def DecodeAsNetworkProtocols(self):
		"""List with network protocols available for capture decode as operation. Need to have an active capture to retrieve the property.

		Returns:
			list(str)
		"""
		return self._get_attribute('decodeAsNetworkProtocols')

	@property
	def DecodeAsTransportProtocols(self):
		"""List with transport protocols available for capture decode as operation. Need to have an active capture to retrieve the property.

		Returns:
			list(str)
		"""
		return self._get_attribute('decodeAsTransportProtocols')

	@property
	def DisplayFiltersControlCapture(self):
		"""Displays the packet filter set inside the control capture that is used to filter the already captured packets

		Returns:
			str
		"""
		return self._get_attribute('displayFiltersControlCapture')
	@DisplayFiltersControlCapture.setter
	def DisplayFiltersControlCapture(self, value):
		self._set_attribute('displayFiltersControlCapture', value)

	@property
	def DisplayFiltersDataCapture(self):
		"""Displays the packet filter set inside the data capture that is used to filter the already captured packets

		Returns:
			str
		"""
		return self._get_attribute('displayFiltersDataCapture')
	@DisplayFiltersDataCapture.setter
	def DisplayFiltersDataCapture(self, value):
		self._set_attribute('displayFiltersDataCapture', value)

	@property
	def HardwareEnabled(self):
		"""If true, enables the capture of data plane traffic. Note that in order for data traffic to be captured, the vport attritbute -rxMode must be set to capture.

		Returns:
			bool
		"""
		return self._get_attribute('hardwareEnabled')
	@HardwareEnabled.setter
	def HardwareEnabled(self, value):
		self._set_attribute('hardwareEnabled', value)

	@property
	def IsCaptureRunning(self):
		"""Indicates if the capture is running.

		Returns:
			bool
		"""
		return self._get_attribute('isCaptureRunning')

	@property
	def IsControlCaptureRunning(self):
		"""Indicates if the control capture is running.

		Returns:
			bool
		"""
		return self._get_attribute('isControlCaptureRunning')

	@property
	def IsDataCaptureRunning(self):
		"""Indicates if the data capture is running.

		Returns:
			bool
		"""
		return self._get_attribute('isDataCaptureRunning')

	@property
	def SliceSize(self):
		"""The size of the capture slice.

		Returns:
			number
		"""
		return self._get_attribute('sliceSize')
	@SliceSize.setter
	def SliceSize(self, value):
		self._set_attribute('sliceSize', value)

	@property
	def SoftwareEnabled(self):
		"""If true, enables the capture of control plane traffic. Note that in order for data traffic to be captured, the vport attritbute -rxMode must be set to capture.

		Returns:
			bool
		"""
		return self._get_attribute('softwareEnabled')
	@SoftwareEnabled.setter
	def SoftwareEnabled(self, value):
		self._set_attribute('softwareEnabled', value)

	@property
	def TriggerPosition(self):
		"""Controls the dividing line within the capture buffer between before trigger data and post trigger data. This control is only useful in triggered mode.

		Returns:
			number
		"""
		return self._get_attribute('triggerPosition')
	@TriggerPosition.setter
	def TriggerPosition(self, value):
		self._set_attribute('triggerPosition', value)

	def update(self, AfterTriggerFilter=None, BeforeTriggerFilter=None, CaptureMode=None, ContinuousFilters=None, ControlActiveCapture=None, ControlBufferBehaviour=None, ControlBufferSize=None, ControlCaptureFilter=None, ControlCaptureTrigger=None, ControlInterfaceType=None, ControlSliceSize=None, DataActiveCapture=None, DataReceiveTimestamp=None, DisplayFiltersControlCapture=None, DisplayFiltersDataCapture=None, HardwareEnabled=None, SliceSize=None, SoftwareEnabled=None, TriggerPosition=None):
		"""Updates a child instance of capture on the server.

		Args:
			AfterTriggerFilter (str(captureAfterTriggerAll|captureAfterTriggerConditionFilter|captureAfterTriggerFilter)): Controls the capture of data after triggering when operating in triggered mode.
			BeforeTriggerFilter (str(captureBeforeTriggerAll|captureBeforeTriggerFilter|captureBeforeTriggerNone)): Controls the capture of data prior to triggering when operating in triggered mode
			CaptureMode (str(captureContinuousMode|captureTriggerMode)): Controls whether data capture is performed in a continuous or triggered mode.
			ContinuousFilters (str(captureContinuousAll|captureContinuousFilter)): Controls the circular buffer behaviour: continuous capture of all received packets or continuous capture of received packets which match the filter conditions applied.
			ControlActiveCapture (str): The name of the active control capture (if any).The active control capture is the last one made on the port by default; but the user can change it using this attribute.
			ControlBufferBehaviour (str(bufferAfterStopCircular|bufferAfterStopNonCircular|bufferLiveCircular|bufferLiveNonCircular)): Sets the control capture buffer behavior.
			ControlBufferSize (number): Sets the size(%) of the ports memory used by the control capture.
			ControlCaptureFilter (str): Controls the dividing line within the capture buffer between before trigger dataand post trigger data. This control is only useful in triggered mode.
			ControlCaptureTrigger (str): This is the control Trigger string.
			ControlInterfaceType (str(anyInterface|specificInterface)): Enables control capture on the desired interfaces.
			ControlSliceSize (number): Sets the size of the control capture slices.
			DataActiveCapture (str): The name of the active data capture (if any). The active data capture is the last one made on the port by default; but the user can change it using this attribute.
			DataReceiveTimestamp (str(chassisUtcTime|hwTimestamp)): Controls whether the data capture packets timestamp are using the chassis UTC time or the HW timestamp.
			DisplayFiltersControlCapture (str): Displays the packet filter set inside the control capture that is used to filter the already captured packets
			DisplayFiltersDataCapture (str): Displays the packet filter set inside the data capture that is used to filter the already captured packets
			HardwareEnabled (bool): If true, enables the capture of data plane traffic. Note that in order for data traffic to be captured, the vport attritbute -rxMode must be set to capture.
			SliceSize (number): The size of the capture slice.
			SoftwareEnabled (bool): If true, enables the capture of control plane traffic. Note that in order for data traffic to be captured, the vport attritbute -rxMode must be set to capture.
			TriggerPosition (number): Controls the dividing line within the capture buffer between before trigger data and post trigger data. This control is only useful in triggered mode.

		Raises:
			ServerError: The server has encountered an uncategorized error condition
		"""
		self._update(locals())

	def DecodeAsApply(self, *args, **kwargs):
		"""Executes the decodeAsApply operation on the server.

		The command forces a re-dissection of all packets based on a filter condition. (similar with Decode As from Wireshark)

		The IxNetwork modeling infrastructure allows for multiple method Signatures with the same name while python does not.
		The following correlates the modeling Signatures to the python *args variable length list:

		decodeAsApply(Arg2:enum, Arg3:enum, Arg4:number, Arg5:string)
			Args:
				args[0] is Arg2 (str(control|data)): The capture type, could be either control or data.
				args[1] is Arg3 (str(link|network|transport)): Specifies the network layer at witch the command should take place.
				args[2] is Arg4 (number): Could be the TCP port for Transport layer (either source or destination), IP protocol for Network layer or Ethertype for Link layer.
				args[3] is Arg5 (str): The protocol name to re-dissect as.

		decodeAsApply(Arg2:enum, Arg3:enum, Arg4:number, Arg5:enum, Arg6:number, Arg7:string)
			Args:
				args[0] is Arg2 (str(control|data)): The capture type, could be either control or data.
				args[1] is Arg3 (str(transport)): The transport layer.
				args[2] is Arg4 (number): The TCP source port.
				args[3] is Arg5 (str(transport)): The transport layer.
				args[4] is Arg6 (number): The TCP destination port.
				args[5] is Arg7 (str): The protocol name to re-dissect as.

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		payload = { "Arg1": self.href }
		for i in range(len(args)): payload['Arg%s' % (i + 2)] = args[i]
		for item in kwargs.items(): payload[item[0]] = item[1]
		return self._execute('decodeAsApply', payload=payload, response_object=None)

	def DecodeAsClear(self, *args, **kwargs):
		"""Executes the decodeAsClear operation on the server.

		The command clears the dissection filter set by DecodeAsApply command.

		decodeAsClear(Arg2:enum)
			Args:
				args[0] is Arg2 (str(control|data)): The capture type, could be either control or data.

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		payload = { "Arg1": self.href }
		for i in range(len(args)): payload['Arg%s' % (i + 2)] = args[i]
		for item in kwargs.items(): payload[item[0]] = item[1]
		return self._execute('decodeAsClear', payload=payload, response_object=None)

	def MergeCapture(self, *args, **kwargs):
		"""Executes the mergeCapture operation on the server.

		The command merges to online captures.

		The IxNetwork modeling infrastructure allows for multiple method Signatures with the same name while python does not.
		The following correlates the modeling Signatures to the python *args variable length list:

		mergeCapture(Arg2:enum, Arg3:href, Arg4:enum, Arg5:string)
			Args:
				args[0] is Arg2 (str(control|data)): The capture type, could be either control or data.
				args[1] is Arg3 (str(None|/api/v1/sessions/1/ixnetwork/vport?deepchild=capture)): The capture object of a port.
				args[2] is Arg4 (str(control|data)): The capture type, could be either control or data.
				args[3] is Arg5 (str): The full path where the resulted merged capture will be saved, the result capture name needs to contain extension also.

		mergeCapture(Arg2:enum, Arg3:string, Arg4:string)
			Args:
				args[0] is Arg2 (str(control|data)): The capture type, could be either control or data.
				args[1] is Arg3 (str): The full path of the offline capture.
				args[2] is Arg4 (str): The full path where the resulted merged capture will be saved, the result capture name needs to contain extension also.

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		payload = { "Arg1": self.href }
		for i in range(len(args)): payload['Arg%s' % (i + 2)] = args[i]
		for item in kwargs.items(): payload[item[0]] = item[1]
		return self._execute('mergeCapture', payload=payload, response_object=None)

	def Start(self, *args, **kwargs):
		"""Executes the start operation on the server.

		This command starts the capture process for a port or group of ports.

		The IxNetwork modeling infrastructure allows for multiple method Signatures with the same name while python does not.
		The following correlates the modeling Signatures to the python *args variable length list:

		start()

		start(Arg2:enum)
			Args:
				args[0] is Arg2 (str(allTraffic|controlTraffic|dataTraffic)): The type of the capture that should be started.

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		payload = { "Arg1": self.href }
		for i in range(len(args)): payload['Arg%s' % (i + 2)] = args[i]
		for item in kwargs.items(): payload[item[0]] = item[1]
		return self._execute('start', payload=payload, response_object=None)

	def Stop(self, *args, **kwargs):
		"""Executes the stop operation on the server.

		This command stops captures for the specified capture configuration.

		The IxNetwork modeling infrastructure allows for multiple method Signatures with the same name while python does not.
		The following correlates the modeling Signatures to the python *args variable length list:

		stop()

		stop(Arg2:enum)
			Args:
				args[0] is Arg2 (str(allTraffic|controlTraffic|dataTraffic)): The capture type.

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		payload = { "Arg1": self.href }
		for i in range(len(args)): payload['Arg%s' % (i + 2)] = args[i]
		for item in kwargs.items(): payload[item[0]] = item[1]
		return self._execute('stop', payload=payload, response_object=None)



# Copyright 1997 - 2018 by IXIA Keysight
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import sys
import os
import ssl
import datetime
import time
import json
import logging
import pkg_resources
from requests import Session, request
from requests.exceptions import ConnectTimeout
from io import BufferedReader
from ixnetwork_restpy.errors import *
from ixnetwork_restpy.files import Files


try:
    basestring
except NameError:
    basestring = str


class Connection(object):
    """Http/Https transport"""
    X_API_KEY = 'X-Api-Key'
    TRACE_NONE = 'none'
    TRACE_REQUEST = 'request'
    TRACE_REQUEST_RESPONSE = 'request_response'
    PLATFORMS = {
        'Jetty': 'linux',
        'SelfHost': 'windows',
        'Microsoft-HTTPAPI/2.0': 'connection_manager'
    }
    def __init__(self, hostname, rest_port, platform, log_file_name=None, ignore_env_proxy=False):
        """ Set the connection parameters to a rest server

        Args:
            hostname (str): hostname or ip address
            rest_port (int, optional): the rest port of the server
            platform (str): 
            log_file_name (str):
            ignore_env_proxy (bool):
        """
        if sys.version < '2.7.9':
            import requests.packages.urllib3
            requests.packages.urllib3.disable_warnings()
        else:
            import urllib3
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        self._headers = {
            Connection.X_API_KEY: None
        }
        self._hostname = hostname
        self._rest_port = rest_port
        self._verify_cert = False
        self._scheme = 'https'
        self._log_file_name = log_file_name
        self._session = Session()

        # setup logging to both console and file if requested
        self._trace = Connection.TRACE_NONE
        if len(logging.getLogger(__name__).handlers) == 0:
            handlers = [logging.StreamHandler(sys.stdout)]
            if log_file_name is not None:
                handlers.append(logging.FileHandler(log_file_name, mode='w'))
            formatter = logging.Formatter(fmt='%(asctime)s [%(name)s] [%(levelname)s] %(message)s', datefmt='%Y-%m-%d %H:%M:%S')
            formatter.converter = time.gmtime
            for handler in handlers:
                handler.setFormatter(formatter)
                logging.getLogger(__name__).addHandler(handler)
            logging.getLogger(__name__).setLevel(logging.INFO)
            logging.getLogger(__name__).info('using python version %s' % sys.version)
            try:
                logging.getLogger(__name__).info('using ixnetwork-restpy version %s' % pkg_resources.get_distribution("ixnetwork-restpy").version)
            except Exception as e:
                logging.getLogger(__name__).warn("ixnetwork-restpy not installed using pip, unable to determine version")

        self._determine_test_tool_platform(platform)

        if self._platform == 'windows':
            self._scheme = 'http'

        if ignore_env_proxy is True:
            self._session.proxies.update({
                'http': None,
                'https': None
            })

    def _determine_test_tool_platform(self, platform):
        self._info('The package will automatically determine the test tool platform and rest_port using the %s address...' % self._hostname)
        if platform is not None:
            self._warn('The `platform` parameter is deprecated and the value `%s` will be ignored.' % platform)
        self._platform = None
        rest_ports = [443, 11009]
        if self._rest_port is not None:
            rest_ports.insert(0, self._rest_port)
        for rest_port in rest_ports:
            for scheme in ['http', 'https']:
                try:
                    url = '%s://%s:%s/api/v1/auth/session' % (scheme, self._hostname, rest_port)
                    payload = json.dumps({'username': '', 'password': ''})
                    headers = {'content-type': 'application/json'}
                    response = self._request(method='POST', url=url, data=payload, headers=headers, verify=self._verify_cert, timeout=2)
                    if response.status_code in [401, 403, 200]:
                        if 'server' not in response.headers:
                            response.headers['server'] = 'Jetty'
                        for server in Connection.PLATFORMS:
                            if server in response.headers['server']:
                                self._platform = Connection.PLATFORMS[server]
                                self._rest_port = rest_port
                                self._info('Connection established to `%s://%s:%s on %s`' % (scheme, self._hostname, self._rest_port, self._platform))
                                return
                    else:
                        raise Exception()
                except Exception as e:
                    self._info('Unable to connect to test tool at %s://%s:%s.' % (scheme, self._hostname, rest_port))
        raise ConnectionError('Unable to connect to %s. Check the ip address and consider using the rest_port parameter.' % self._hostname)

    @property
    def trace(self):
        """str: Trace all requests and responses."""
        return self._trace
    @trace.setter
    def trace(self, value):
        if value not in [Connection.TRACE_NONE, Connection.TRACE_REQUEST, Connection.TRACE_REQUEST_RESPONSE]:
            raise ValueError('the value %s is an incorrect Trace level' % value)
        self._trace = value
        if self._trace == Connection.TRACE_NONE:
            logging.getLogger(__name__).setLevel(logging.INFO)
        if self._trace in [Connection.TRACE_REQUEST, Connection.TRACE_REQUEST_RESPONSE]:
            logging.getLogger(__name__).setLevel(logging.DEBUG)

    @property
    def platform(self):
        return self._platform

    @property
    def hostname(self):
        return self._hostname

    @property
    def rest_port(self):
        return self._rest_port

    @property
    def scheme(self):
        return self._scheme

    @property
    def log_file_name(self):
        return self._log_file_name

    @property
    def x_api_key(self):
        """str: Get/Set the x-api-key header value."""
        return self._headers[Connection.X_API_KEY]
    @x_api_key.setter
    def x_api_key(self, value):
        self._headers[Connection.X_API_KEY] = value

    def _request(self, **kwargs):
        return self._session.request(**kwargs)

    def _read(self, url):
        return self._send_recv('GET', url)

    def _create(self, url, payload):
        return self._send_recv('POST', url, payload)

    def _update(self, url, payload):
        return self._send_recv('PATCH', url, payload)

    def _delete(self, url, payload=None):
        return self._send_recv('DELETE', url, payload)

    def _execute(self, url, payload):
        return self._send_recv('POST', url, payload)

    def _options(self, url):
        return self._send_recv('OPTIONS', url)

    def _print_request(self, method, url, payload=None):
        if self._trace in [Connection.TRACE_REQUEST, Connection.TRACE_REQUEST_RESPONSE]:
            data = ''
            if payload is not None:
                data = payload[0:128]
                if len(payload) > 128:
                    data += ' ...'
            logging.getLogger(__name__).debug('%s %s %s' % (method, url, data))
    
    def _print_response(self, response):
        if self._trace == Connection.TRACE_REQUEST_RESPONSE:
            data = ''
            if response.status_code >= 400:
                data = response.raw.data
            else:
                data = response.raw.data[0:128]
            if len(response.raw.data) > 128:
                data += ' ...'
            logging.getLogger(__name__).debug('%s %s %s' % (response.status_code, response.reason, data))
    
    def _info(self, message):
        logging.getLogger(__name__).info(message)

    def _warn(self, message):
        logging.getLogger(__name__).warn(message)

    def _debug(self, message):
        logging.getLogger(__name__).debug(message)

    def _normalize_url(self, url):
        connection = '%s://%s:%s' % (self._scheme, self._hostname, self._rest_port)
        if url.startswith(self._scheme) == False:
            url = '%s/%s' % (connection, url.strip('/'))
        
        path_start = url.find('://') + 3
        url = '%s%s' % (url[0:path_start], url[path_start:].replace('//', '/'))
        return (connection, url)

    def _get_file(self, url, remote_filename, remote_directory=None, local_filename=None, local_directory=None):
        headers = self._headers
        url = '%s/files?filename=%s' % (url, remote_filename)
        connection, url = self._normalize_url(url)
        if remote_directory is not None:
            url = '%s&absolute=%s' % (url, remote_directory)
        response = self._session.request('GET', url, headers=headers, verify=self._verify_cert)
        if response.status_code == 200:
            if local_filename is None:
                local_filename = remote_filename
            if local_directory is not None:
                local_filename = os.path.join(local_directory, local_filename)
            local_filename = os.path.normpath(local_filename)
            try:
                with open(local_filename, 'wb') as fid:
                    fid.write(response.content)
            except Exception as e:
                self._info('cwd:%s filename:%s exception:%s' % (os.getcwd(), local_filename, e))
                raise e
            return local_filename
        else:
            self._process_response_status_code(response) 

    def _put_file(self, url, local_filename, remote_filename=None):
        headers = self._headers
        if remote_filename is None:
            remote_filename = os.path.basename(local_filename)
        url = '%s/files?filename=%s' % (url, remote_filename)
        connection, url = self._normalize_url(url)
        with open(local_filename, 'rb') as fid:
            data = fid.read()
        response = self._session.request('POST', url, headers=headers, data=data, verify=self._verify_cert)
        if response.status_code == 201:
            return response.json()
        else:
            self._process_response_status_code(response)

    def _process_response_status_code(self, response):
        if response.status_code == 400:
            raise BadRequestError(response)
        elif response.status_code == 401:
            raise UnauthorizedError(response)
        elif response.status_code == 404:
            raise NotFoundError(response)
        elif response.status_code == 409:
            raise ResourceInUseError(response)
        else:
            raise ServerError(response)

    def _send_recv(self, method, url, payload=None):
        headers = self._headers
        headers['Content-Type'] = 'application/octet-stream'
        connection, url = self._normalize_url(url)

        data = payload
        if payload is not None:
            if isinstance(payload, dict) or isinstance(payload, list):
                headers['Content-Type'] = 'application/json'
                data = json.dumps(payload)
            elif isinstance(payload, Files):                          
                headers['Content-Type'] = 'application/octet-stream'
                if os.path.isfile(payload.file_path):
                    with open(payload.file_path, 'rb') as fid:
                        data = fid.read()
                else:
                    response = self._session.request('GET', url.replace('filename=', 'filter='), headers=headers, verify=self._verify_cert, allow_redirects=False)
                    if response.status_code == 200:
                        return
                    data = ''
            elif isinstance(payload, basestring):
                headers['Content-Type'] = 'application/json'
                data = payload

        self._print_request(method, url, None if isinstance(payload, Files) else data)
        response = self._request(method=method, url=url, data=data, headers=headers, verify=self._verify_cert, allow_redirects=False)
        self._print_response(response)
        
        if str(response.status_code).startswith('3'):
            url = response.headers['location']
            if url.find('://') != -1:
                self._scheme = url[:url.find('://')]
                self._hostname = url[url.find('://')+3:url.find('/', url.find('://')+3)]
                if self._scheme == 'https':
                    self._rest_port = 443
                host_pieces = self._hostname.split(':')
                if len(host_pieces) > 1:
                    self._hostname = host_pieces[0]
                    self._rest_port = host_pieces[1]
            else:
                url = '%s://%s:%s%s' % (self._scheme, self._hostname, self._rest_port, url)
            self._print_request(method, url, data)
            response = self._session.request(method, url, data=data, headers=headers, verify=self._verify_cert, allow_redirects=False)
            self._print_response(response)

        if response.status_code == 202:
            while True:
                async_status = response.json()
                if 'state' not in async_status.keys():
                    break
                state = async_status['state']
                if state == 'IN_PROGRESS':
                    time.sleep(1)
                    state_url = async_status['url']
                    if state_url.startswith(self._scheme) == False:
                        state_url = '%s/%s' % (connection, state_url.strip('/'))
                    self._print_request('GET', state_url)
                    response = self._session.request('GET', state_url, headers=headers, verify=self._verify_cert)
                    self._print_response(response)
                elif state == 'SUCCESS':
                    if 'result' in async_status.keys():
                        return async_status['result']
                    else:
                        return None
                elif state in ['ERROR', 'EXCEPTION']:
                    raise ServerError(response)
                elif self._platform == 'connection_manager':
                    if async_status['state'] == 'ACTIVE':
                        return response.json()
                    time.sleep(3)
                    base_url, state_url = self._normalize_url(async_status['links'][0]['href'])
                    self._print_request('GET', state_url)
                    response = self._session.request('GET', state_url, headers=headers, verify=self._verify_cert)
                    self._print_response(response)
                elif async_status['message'] is not None and 'API CONTENTION' in async_status['message']:
                    raise ResourceInUseError(response)
                else:
                    raise ServerError(response) 
        
        while(response.status_code == 409):
            time.sleep(6)
            response = self._session.request(method, url, data=data, headers=headers, verify=self._verify_cert)

        if response.status_code == 204:
            return None
        elif str(response.status_code).startswith('2') is True:
            if response.status_code == 201 and 'links' in response.json().keys():
                href = response.json()['links'][0]['href']
                return self._send_recv('GET', href)
            if response.headers.get('Content-Type'):
                if 'application/json' in response.headers['Content-Type']:
                   return response.json()
            return None
        else:
            self._process_response_status_code(response)





# MIT LICENSE
#
# Copyright 1997 - 2019 by IXIA Keysight
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
from ixnetwork_restpy.base import Base
from ixnetwork_restpy.files import Files


class SpbNetworkRange(Base):
	"""The SpbNetworkRange class encapsulates a user managed spbNetworkRange node in the ixnetwork hierarchy.

	An instance of the class can be obtained by accessing the SpbNetworkRange property from a parent instance.
	The internal properties list will be empty when the property is accessed and is populated from the server using the find method.
	The internal properties list can be managed by the user by using the add and remove methods.
	"""

	_SDM_NAME = 'spbNetworkRange'

	def __init__(self, parent):
		super(SpbNetworkRange, self).__init__(parent)

	@property
	def SpbOutsideLinks(self):
		"""An instance of the SpbOutsideLinks class.

		Returns:
			obj(ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.protocols.isis.spboutsidelinks.SpbOutsideLinks)

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		from ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.protocols.isis.spboutsidelinks import SpbOutsideLinks
		return SpbOutsideLinks(self)

	@property
	def SpbmNodeTopologyRange(self):
		"""An instance of the SpbmNodeTopologyRange class.

		Returns:
			obj(ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.protocols.isis.spbmnodetopologyrange.SpbmNodeTopologyRange)

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		from ixnetwork_restpy.testplatform.sessions.ixnetwork.vport.protocols.isis.spbmnodetopologyrange import SpbmNodeTopologyRange
		return SpbmNodeTopologyRange(self)

	@property
	def EnableAdvertiseNetworkRange(self):
		"""If true, this SPB ISIS Network Range is advertised.

		Returns:
			bool
		"""
		return self._get_attribute('enableAdvertiseNetworkRange')
	@EnableAdvertiseNetworkRange.setter
	def EnableAdvertiseNetworkRange(self, value):
		self._set_attribute('enableAdvertiseNetworkRange', value)

	@property
	def EnableHostName(self):
		"""If true, the host name of the router is activated.

		Returns:
			bool
		"""
		return self._get_attribute('enableHostName')
	@EnableHostName.setter
	def EnableHostName(self, value):
		self._set_attribute('enableHostName', value)

	@property
	def EntryColumn(self):
		"""The value is used in combination to specify which virtual router in the Network Range is connected to the current ISIS L2/L3 Router.

		Returns:
			number
		"""
		return self._get_attribute('entryColumn')
	@EntryColumn.setter
	def EntryColumn(self, value):
		self._set_attribute('entryColumn', value)

	@property
	def EntryRow(self):
		"""The value is used in combination to specify which virtual router in the Network Range is connected to the current ISIS L2/L3 Router.

		Returns:
			number
		"""
		return self._get_attribute('entryRow')
	@EntryRow.setter
	def EntryRow(self, value):
		self._set_attribute('entryRow', value)

	@property
	def HostNamePrefix(self):
		"""The host name prefix information.

		Returns:
			str
		"""
		return self._get_attribute('hostNamePrefix')
	@HostNamePrefix.setter
	def HostNamePrefix(self, value):
		self._set_attribute('hostNamePrefix', value)

	@property
	def InterfaceMetric(self):
		"""The metric cost associated with this emulated SPB ISIS router.

		Returns:
			number
		"""
		return self._get_attribute('interfaceMetric')
	@InterfaceMetric.setter
	def InterfaceMetric(self, value):
		self._set_attribute('interfaceMetric', value)

	@property
	def NoOfColumns(self):
		"""The value is used in combination to create a matrix (grid) for an emulated network range of the following size: The # Rows multiplied the # Cols = Number of routers in this Network Range. (For example, 3 Rows x 3 Columns = 9 Routers).

		Returns:
			number
		"""
		return self._get_attribute('noOfColumns')
	@NoOfColumns.setter
	def NoOfColumns(self, value):
		self._set_attribute('noOfColumns', value)

	@property
	def NoOfRows(self):
		"""The value is used in combination to create a matrix (grid) for an emulated network range of the following size: The # Rows multiplied the # Cols = Number of routers in this Network Range. (For example, 3 Rows x 3 Columns = 9 Routers).

		Returns:
			number
		"""
		return self._get_attribute('noOfRows')
	@NoOfRows.setter
	def NoOfRows(self, value):
		self._set_attribute('noOfRows', value)

	@property
	def StartSystemId(self):
		"""The System ID assigned to the starting SPB ISIS router in this network range. The default is 00 00 00 00 00 00.

		Returns:
			str
		"""
		return self._get_attribute('startSystemId')
	@StartSystemId.setter
	def StartSystemId(self, value):
		self._set_attribute('startSystemId', value)

	@property
	def SystemIdIncrementBy(self):
		"""This is used when more than one router is to be emulated. The increment value is added to the previous System ID for each additional emulated router in this network range.

		Returns:
			str
		"""
		return self._get_attribute('systemIdIncrementBy')
	@SystemIdIncrementBy.setter
	def SystemIdIncrementBy(self, value):
		self._set_attribute('systemIdIncrementBy', value)

	def update(self, EnableAdvertiseNetworkRange=None, EnableHostName=None, EntryColumn=None, EntryRow=None, HostNamePrefix=None, InterfaceMetric=None, NoOfColumns=None, NoOfRows=None, StartSystemId=None, SystemIdIncrementBy=None):
		"""Updates a child instance of spbNetworkRange on the server.

		Args:
			EnableAdvertiseNetworkRange (bool): If true, this SPB ISIS Network Range is advertised.
			EnableHostName (bool): If true, the host name of the router is activated.
			EntryColumn (number): The value is used in combination to specify which virtual router in the Network Range is connected to the current ISIS L2/L3 Router.
			EntryRow (number): The value is used in combination to specify which virtual router in the Network Range is connected to the current ISIS L2/L3 Router.
			HostNamePrefix (str): The host name prefix information.
			InterfaceMetric (number): The metric cost associated with this emulated SPB ISIS router.
			NoOfColumns (number): The value is used in combination to create a matrix (grid) for an emulated network range of the following size: The # Rows multiplied the # Cols = Number of routers in this Network Range. (For example, 3 Rows x 3 Columns = 9 Routers).
			NoOfRows (number): The value is used in combination to create a matrix (grid) for an emulated network range of the following size: The # Rows multiplied the # Cols = Number of routers in this Network Range. (For example, 3 Rows x 3 Columns = 9 Routers).
			StartSystemId (str): The System ID assigned to the starting SPB ISIS router in this network range. The default is 00 00 00 00 00 00.
			SystemIdIncrementBy (str): This is used when more than one router is to be emulated. The increment value is added to the previous System ID for each additional emulated router in this network range.

		Raises:
			ServerError: The server has encountered an uncategorized error condition
		"""
		self._update(locals())

	def add(self, EnableAdvertiseNetworkRange=None, EnableHostName=None, EntryColumn=None, EntryRow=None, HostNamePrefix=None, InterfaceMetric=None, NoOfColumns=None, NoOfRows=None, StartSystemId=None, SystemIdIncrementBy=None):
		"""Adds a new spbNetworkRange node on the server and retrieves it in this instance.

		Args:
			EnableAdvertiseNetworkRange (bool): If true, this SPB ISIS Network Range is advertised.
			EnableHostName (bool): If true, the host name of the router is activated.
			EntryColumn (number): The value is used in combination to specify which virtual router in the Network Range is connected to the current ISIS L2/L3 Router.
			EntryRow (number): The value is used in combination to specify which virtual router in the Network Range is connected to the current ISIS L2/L3 Router.
			HostNamePrefix (str): The host name prefix information.
			InterfaceMetric (number): The metric cost associated with this emulated SPB ISIS router.
			NoOfColumns (number): The value is used in combination to create a matrix (grid) for an emulated network range of the following size: The # Rows multiplied the # Cols = Number of routers in this Network Range. (For example, 3 Rows x 3 Columns = 9 Routers).
			NoOfRows (number): The value is used in combination to create a matrix (grid) for an emulated network range of the following size: The # Rows multiplied the # Cols = Number of routers in this Network Range. (For example, 3 Rows x 3 Columns = 9 Routers).
			StartSystemId (str): The System ID assigned to the starting SPB ISIS router in this network range. The default is 00 00 00 00 00 00.
			SystemIdIncrementBy (str): This is used when more than one router is to be emulated. The increment value is added to the previous System ID for each additional emulated router in this network range.

		Returns:
			self: This instance with all currently retrieved spbNetworkRange data using find and the newly added spbNetworkRange data available through an iterator or index

		Raises:
			ServerError: The server has encountered an uncategorized error condition
		"""
		return self._create(locals())

	def remove(self):
		"""Deletes all the spbNetworkRange data in this instance from server.

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		self._delete()

	def find(self, EnableAdvertiseNetworkRange=None, EnableHostName=None, EntryColumn=None, EntryRow=None, HostNamePrefix=None, InterfaceMetric=None, NoOfColumns=None, NoOfRows=None, StartSystemId=None, SystemIdIncrementBy=None):
		"""Finds and retrieves spbNetworkRange data from the server.

		All named parameters support regex and can be used to selectively retrieve spbNetworkRange data from the server.
		By default the find method takes no parameters and will retrieve all spbNetworkRange data from the server.

		Args:
			EnableAdvertiseNetworkRange (bool): If true, this SPB ISIS Network Range is advertised.
			EnableHostName (bool): If true, the host name of the router is activated.
			EntryColumn (number): The value is used in combination to specify which virtual router in the Network Range is connected to the current ISIS L2/L3 Router.
			EntryRow (number): The value is used in combination to specify which virtual router in the Network Range is connected to the current ISIS L2/L3 Router.
			HostNamePrefix (str): The host name prefix information.
			InterfaceMetric (number): The metric cost associated with this emulated SPB ISIS router.
			NoOfColumns (number): The value is used in combination to create a matrix (grid) for an emulated network range of the following size: The # Rows multiplied the # Cols = Number of routers in this Network Range. (For example, 3 Rows x 3 Columns = 9 Routers).
			NoOfRows (number): The value is used in combination to create a matrix (grid) for an emulated network range of the following size: The # Rows multiplied the # Cols = Number of routers in this Network Range. (For example, 3 Rows x 3 Columns = 9 Routers).
			StartSystemId (str): The System ID assigned to the starting SPB ISIS router in this network range. The default is 00 00 00 00 00 00.
			SystemIdIncrementBy (str): This is used when more than one router is to be emulated. The increment value is added to the previous System ID for each additional emulated router in this network range.

		Returns:
			self: This instance with matching spbNetworkRange data retrieved from the server available through an iterator or index

		Raises:
			ServerError: The server has encountered an uncategorized error condition
		"""
		return self._select(locals())

	def read(self, href):
		"""Retrieves a single instance of spbNetworkRange data from the server.

		Args:
			href (str): An href to the instance to be retrieved

		Returns:
			self: This instance with the spbNetworkRange data from the server available through an iterator or index

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		return self._read(href)



# MIT LICENSE
#
# Copyright 1997 - 2019 by IXIA Keysight
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
from ixnetwork_restpy.base import Base
from ixnetwork_restpy.files import Files


class TunnelHeadTrafficEndPoint(Base):
	"""The TunnelHeadTrafficEndPoint class encapsulates a user managed tunnelHeadTrafficEndPoint node in the ixnetwork hierarchy.

	An instance of the class can be obtained by accessing the TunnelHeadTrafficEndPoint property from a parent instance.
	The internal properties list will be empty when the property is accessed and is populated from the server using the find method.
	The internal properties list can be managed by the user by using the add and remove methods.
	"""

	_SDM_NAME = 'tunnelHeadTrafficEndPoint'

	def __init__(self, parent):
		super(TunnelHeadTrafficEndPoint, self).__init__(parent)

	@property
	def EndPointType(self):
		"""IPv4/IPv6 address. It has the same values as of IP Type for traffic item in parent Tail Range.

		Returns:
			str(ipv4|ipv6|17|18)
		"""
		return self._get_attribute('endPointType')
	@EndPointType.setter
	def EndPointType(self, value):
		self._set_attribute('endPointType', value)

	@property
	def InsertExplicitTrafficItem(self):
		"""DEPRECATED NOT DEFINED

		Returns:
			bool
		"""
		return self._get_attribute('insertExplicitTrafficItem')
	@InsertExplicitTrafficItem.setter
	def InsertExplicitTrafficItem(self, value):
		self._set_attribute('insertExplicitTrafficItem', value)

	@property
	def InsertIpv6ExplicitNull(self):
		"""DEPRECATED This causes an IPv6 Explicit NULL to be inserted as the innermost label in addition to learned label when trying to generate IPv6 traffic over the IPv4 LSP. The purpose of this is to route the traffic to the IPv6 Protocol Stack at the egress for routing towards the IPv6 destination.

		Returns:
			bool
		"""
		return self._get_attribute('insertIpv6ExplicitNull')
	@InsertIpv6ExplicitNull.setter
	def InsertIpv6ExplicitNull(self, value):
		self._set_attribute('insertIpv6ExplicitNull', value)

	@property
	def IpCount(self):
		"""DEPRECATED Allows value greater than or equal to Tunnel Head IP Count (1 by default). This can be used to simulate traffic from multiple source endpoints to be sent over the LSPs originated from the Head Range.

		Returns:
			number
		"""
		return self._get_attribute('ipCount')
	@IpCount.setter
	def IpCount(self, value):
		self._set_attribute('ipCount', value)

	@property
	def IpStart(self):
		"""DEPRECATED The Source IP address, one of IPv4 or IPv6, to be used for traffic to be sent over LSPs from the Head End Point.

		Returns:
			str
		"""
		return self._get_attribute('ipStart')
	@IpStart.setter
	def IpStart(self, value):
		self._set_attribute('ipStart', value)

	def update(self, EndPointType=None, InsertExplicitTrafficItem=None, InsertIpv6ExplicitNull=None, IpCount=None, IpStart=None):
		"""Updates a child instance of tunnelHeadTrafficEndPoint on the server.

		Args:
			EndPointType (str(ipv4|ipv6|17|18)): IPv4/IPv6 address. It has the same values as of IP Type for traffic item in parent Tail Range.
			InsertExplicitTrafficItem (bool): NOT DEFINED
			InsertIpv6ExplicitNull (bool): This causes an IPv6 Explicit NULL to be inserted as the innermost label in addition to learned label when trying to generate IPv6 traffic over the IPv4 LSP. The purpose of this is to route the traffic to the IPv6 Protocol Stack at the egress for routing towards the IPv6 destination.
			IpCount (number): Allows value greater than or equal to Tunnel Head IP Count (1 by default). This can be used to simulate traffic from multiple source endpoints to be sent over the LSPs originated from the Head Range.
			IpStart (str): The Source IP address, one of IPv4 or IPv6, to be used for traffic to be sent over LSPs from the Head End Point.

		Raises:
			ServerError: The server has encountered an uncategorized error condition
		"""
		self._update(locals())

	def add(self, EndPointType=None, InsertExplicitTrafficItem=None, InsertIpv6ExplicitNull=None, IpCount=None, IpStart=None):
		"""Adds a new tunnelHeadTrafficEndPoint node on the server and retrieves it in this instance.

		Args:
			EndPointType (str(ipv4|ipv6|17|18)): IPv4/IPv6 address. It has the same values as of IP Type for traffic item in parent Tail Range.
			InsertExplicitTrafficItem (bool): NOT DEFINED
			InsertIpv6ExplicitNull (bool): This causes an IPv6 Explicit NULL to be inserted as the innermost label in addition to learned label when trying to generate IPv6 traffic over the IPv4 LSP. The purpose of this is to route the traffic to the IPv6 Protocol Stack at the egress for routing towards the IPv6 destination.
			IpCount (number): Allows value greater than or equal to Tunnel Head IP Count (1 by default). This can be used to simulate traffic from multiple source endpoints to be sent over the LSPs originated from the Head Range.
			IpStart (str): The Source IP address, one of IPv4 or IPv6, to be used for traffic to be sent over LSPs from the Head End Point.

		Returns:
			self: This instance with all currently retrieved tunnelHeadTrafficEndPoint data using find and the newly added tunnelHeadTrafficEndPoint data available through an iterator or index

		Raises:
			ServerError: The server has encountered an uncategorized error condition
		"""
		return self._create(locals())

	def remove(self):
		"""Deletes all the tunnelHeadTrafficEndPoint data in this instance from server.

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		self._delete()

	def find(self, EndPointType=None, InsertExplicitTrafficItem=None, InsertIpv6ExplicitNull=None, IpCount=None, IpStart=None):
		"""Finds and retrieves tunnelHeadTrafficEndPoint data from the server.

		All named parameters support regex and can be used to selectively retrieve tunnelHeadTrafficEndPoint data from the server.
		By default the find method takes no parameters and will retrieve all tunnelHeadTrafficEndPoint data from the server.

		Args:
			EndPointType (str(ipv4|ipv6|17|18)): IPv4/IPv6 address. It has the same values as of IP Type for traffic item in parent Tail Range.
			InsertExplicitTrafficItem (bool): NOT DEFINED
			InsertIpv6ExplicitNull (bool): This causes an IPv6 Explicit NULL to be inserted as the innermost label in addition to learned label when trying to generate IPv6 traffic over the IPv4 LSP. The purpose of this is to route the traffic to the IPv6 Protocol Stack at the egress for routing towards the IPv6 destination.
			IpCount (number): Allows value greater than or equal to Tunnel Head IP Count (1 by default). This can be used to simulate traffic from multiple source endpoints to be sent over the LSPs originated from the Head Range.
			IpStart (str): The Source IP address, one of IPv4 or IPv6, to be used for traffic to be sent over LSPs from the Head End Point.

		Returns:
			self: This instance with matching tunnelHeadTrafficEndPoint data retrieved from the server available through an iterator or index

		Raises:
			ServerError: The server has encountered an uncategorized error condition
		"""
		return self._select(locals())

	def read(self, href):
		"""Retrieves a single instance of tunnelHeadTrafficEndPoint data from the server.

		Args:
			href (str): An href to the instance to be retrieved

		Returns:
			self: This instance with the tunnelHeadTrafficEndPoint data from the server available through an iterator or index

		Raises:
			NotFoundError: The requested resource does not exist on the server
			ServerError: The server has encountered an uncategorized error condition
		"""
		return self._read(href)
